clc;
clear; close all;
addpath('C:\Users\NOTADAS\Google Drive\PhD\MATLAB\casadi-matlabR2016a-v3.3.0')
import casadi.*


swirl % Figure 7 in paper

%% casadi definitions
X = SX.sym('X',35,1);
Xdot = SX.sym('Xdot',30,1);
n_fb_dot = Xdot(1:10,1);
n_d_dot = Xdot(11:20,1);
n_lb_dot = Xdot(21:30,1);
U = SX.sym('U',3,1); % Inputs
D = SX.sym('D',2,1); % Inputs

%% defining casadi function for the model

n_x = 10; % Number of vertically discretized control volumes, hence n_x + 2 nodes one each at the two boundaries x = 0 and x = L
% Control volumes start from the bottom. Hence x = 0 to x = L/n_x is the first control volume
scaling_factor = 1e10;
x = X;
u = U;

F_in = D(1);
ppm_in = D(2);
S = 0.01*F_in;
F_in = F_in/3600;
e_in = ppm_in*1e-6;

% Allocation of states
n_fb = x(1:n_x)*scaling_factor;
n_d = x(n_x+1:2*n_x)*scaling_factor;
n_lb = x(2*n_x+1:3*n_x)*scaling_factor;
alpha_l = x(3*n_x+1);
P = x(3*n_x+2)*1e5;
n_d_bottom = x(3*n_x+3)*scaling_factor;
alpha_gas_top = x(3*n_x+4);
alpha_oil_top = x(3*n_x+5);

% Dimensions
V = 3.2; % m^3 volume of the CFU main separation zone volume
R = 1.26/2; % Radius of CFU vessel in meter
L = V/pi/R^2;
dL = L/(n_x+2);

% Operating data
db = 200e-6; % Gas bubble size in meter
dd = 25e-6; % Oil droplet size in meter
V_fb = pi*db^3/6; % volume of the bubble at P_bottom
V_d = pi*dd^3/6;
Ec = (dd/db)^2; % Collision efficiency due to Flint and Howarth
k =  4e-11; % Loading rate constant [m^-3s^-1] Tuning parameter

% Fluid data and constants
rho_w = 1000;
rho_o = 900;
P_supply = 2e5;
P_atm = 1e5;
M_g = 16e-3; % Molecular mass of methane in kg/mol
R_g = 8.314; % Universal gas cosntant
T = 300; % Operating Temperature in Kelvin
rho_g = P * M_g / R_g/T;

mu_w = 8.9e-4;
g = 9.81;
F_float = u(1)/3600;
F_out = u(2)/3600;

% Velocities
v_bulk = F_out/pi/R^2/alpha_l;
v_fb = (rho_w - rho_g)*g*db^2/18/mu_w -v_bulk;  % absolute up velocity
v_d  = -(rho_w - rho_o)*g*dd^2/18/mu_w + v_bulk; % absolute down velocity
v_lb = ((rho_w-rho_g)*V_fb+(rho_w-rho_o)*V_d)*g/3/pi/mu_w/((db^3+dd^3)^(1/3)) - v_bulk; % absolute up velocity


% droplets at bottom control volume below everything else. in this control
% volume we dont have differential equations for any thing else, no loading
% happening here

Split = 0.4082*(S-0.7)^2+0.8;
e_in_as = Split*e_in;
F_split_down = F_in*(1-e_in)/(1-e_in_as);
F_split_oil = F_in*e_in - F_split_down*e_in_as;

n_d_dot_bottom =   (v_d*n_d(1) - (F_out/pi/R^2)*n_d_bottom)/dL;

% Bottom boundary conditions
i = 1;
load_rate = (k*Ec*n_fb(i)*n_d(i));
n_fb_dot(i) =  (F_float*(P_supply/P)/V_fb/(pi*R^2)-n_fb(i)*v_fb)/dL  - load_rate;
n_d_dot(i) =   v_d*(n_d(i+1)-n_d(i))/dL - load_rate;
n_lb_dot(i) =  (0-n_lb(i)*v_lb)/dL   + load_rate;

% Differential equations for internal control volumes
for i = 2:n_x-1
load_rate = k*Ec*n_fb(i)*n_d(i);
n_fb_dot(i) =  v_fb*(n_fb(i-1)-n_fb(i))/dL    -load_rate;
n_d_dot(i) =   v_d*((n_d(i+1))-(n_d(i)))/dL   -load_rate;
n_lb_dot(i) =  v_lb*(n_lb(i-1)-n_lb(i))/dL    +load_rate;
end

% Top boundary conditions
i = n_x;
load_rate = k*Ec*n_fb(i)*n_d(i);
n_fb_dot(i) = v_fb*(n_fb(i-1)-n_fb(i))/dL  -load_rate;
n_d_dot(i) = (F_split_down*e_in_as/V_d/(pi*R^2)-n_d(i)*v_d)/dL - load_rate;
n_lb_dot(i) = v_lb*(n_lb(i-1)-n_lb(i))/dL  + load_rate;

F_reject = u(3)/3600;
F_reject_liq = F_reject*(1-alpha_gas_top);
alpha_l_dot = (F_in - F_out - F_reject_liq)/V;
V_l_dot = alpha_l_dot*V;
F_gas_out = alpha_gas_top*F_reject;
V_g = V*(1-alpha_l);
P_dot = (P*V_l_dot + (P_supply*F_float - P*F_gas_out))/V_g;

F_reject_oil = alpha_oil_top*F_reject;
alpha_gt_dot =    (V_fb*(v_fb*n_fb(n_x)+v_lb*n_lb(n_x))     - F_reject/(pi*R^2)*alpha_gas_top)/dL;


alpha_ot_dot =   (V_d*v_lb*n_lb(n_x) + (F_split_oil - F_reject*alpha_oil_top)/(pi*R^2))/dL;
ppm_out = n_d_bottom*V_d*1e6;
F_reject_water = F_reject*(1-alpha_gas_top - alpha_oil_top)*3600;
m_dot_flotation = P_supply*F_float*3600*M_g/R_g/T;

% Purging the differential equations
dots = [n_fb_dot/scaling_factor;n_d_dot/scaling_factor;n_lb_dot/scaling_factor; alpha_l_dot; P_dot/1e5; n_d_dot_bottom/scaling_factor; alpha_gt_dot; alpha_ot_dot]; 
outputs = [e_in_as*1e6;  ppm_out; P/1e5; F_reject_water; F_out*3600; m_dot_flotation; alpha_gas_top; alpha_oil_top; F_reject*3600];


f = Function('f', {X,U,D}, {dots});
y = Function('y', {X,U,D}, {outputs});


%% Steady state
x0 = [2.5459    2.3698    2.1631    1.9271    1.6655    1.3851    1.0953    1.8067    1.5295    1.2728    0.1982    0.2503    0.3128    0.3861    0.4699    0.5627    0.6622    0.7651    0.8675    0.9659    0.1470    0.3235    0.4305    0.5670    1.0290    1.098    1.2001    1.2893    2.0669    2.241    0.9    1.4559    0.2315    0.8   1e-3]';
F_in =70; ppm_in = 150;
D_val = [F_in; ppm_in]; 

U_val = [1.485*5/2; 0.99*F_in;5.6];  
U_initial = U_val;

% IPOPT
prob = struct('f', [0 0 0 0 0 0 0 0 0]*y(X,U,D),'x',[X;U],'g',[f(X,U,D);y(X,U,D)],'p',D);
solver = nlpsol('solver', 'ipopt', prob);
sol = solver('p',D_val,'x0',[x0;U_val], 'lbx', [zeros(31,1);P_atm/1e5;zeros(3,1);U_val], 'ubx', [inf(30,1);1;P_supply/1e5;inf(3,1);U_val], 'lbg', [zeros(35,1);0;0;1;0;0.9*F_in;0;0;0;0], 'ubg', [zeros(35,1);inf;inf;5;0.1*F_in;F_in;inf;1;1;inf]); 
x0_IPOPT = full(sol.x)
y = y(x0_IPOPT(1:35),x0_IPOPT(36:38),D_val)
f(x0_IPOPT(1:35),x0_IPOPT(36:38),D_val)
x0 = x0_IPOPT(1:35); U_val = x0_IPOPT(36:38);
x_initial1 = x0;
column3= [flipud([x0_IPOPT(1:10) x0_IPOPT(11:20) x0_IPOPT(21:30)]); 0 x0_IPOPT(33) 0 ]
yout = [y(3);x0(31);y(2);y(end-1);y(end-2)]

OL_CL_PID